use gtk::prelude::*;

use relm4::factory::FactoryVecDeque;
use relm4::RelmListBoxExt;
use relm4::{gtk, ComponentParts, ComponentSender, SimpleComponent};

use crate::sheet::Sheet;

use super::sheet_model::{OnQueryUpdate, SheetModel};

pub struct SheetListingModel {
    sheets: FactoryVecDeque<SheetModel>,
}

#[derive(Debug)]
pub enum SheetListingInput {
    Query(String),
    ListBoxRowClicked(i32),
}

#[derive(Debug)]
pub struct SheetModelSelected {
    pub sheet: Sheet,
}

#[relm4::component(pub)]
impl SimpleComponent for SheetListingModel {
    type Init = Vec<Sheet>;
    type Input = SheetListingInput;
    type Output = SheetModelSelected;

    view! {
        #[root]
        gtk::Box {
            // set_orientation: gtk::Orientation::Vertical,

            model.sheets.widget() -> &relm4::gtk::ListBox {
                set_hexpand: true,
                // set_orientation: gtk::Orientation::Vertical,
                // set_spacing: 5,
                connect_row_activated[sender] => move |list_box, row| {
                    let index = list_box.index_of_child(row).unwrap();
                    sender.input(SheetListingInput::ListBoxRowClicked(index));
                },
            },
        }
    }

    fn init(
        init: Self::Init,
        root: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> ComponentParts<Self> {
        let mut sheets = FactoryVecDeque::new(gtk::ListBox::default(), sender.input_sender());
        for sheet_model_type in init {
            sheets.guard().push_back(sheet_model_type);
        }

        let model = SheetListingModel { sheets };
        let widgets = view_output!();
        ComponentParts { model, widgets }
    }

    fn update(&mut self, message: Self::Input, sender: ComponentSender<Self>) {
        match message {
            SheetListingInput::Query(query) => {
                self.sheets.broadcast(OnQueryUpdate { query });
            }
            SheetListingInput::ListBoxRowClicked(index) => {
                let sheet_model = self.sheets.get(index as usize).unwrap();
                sender
                    .output(SheetModelSelected {
                        sheet: sheet_model.sheet.clone(),
                    })
                    .unwrap();
            }
        }
    }
}
